/* $USAGI: calc_hmac.c,v 1.2 2001/09/08 08:32:11 yoshfuji Exp $ */

#define __NO_VERSION__
#include <linux/module.h>
#include <linux/version.h>
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#define DEBUG 1
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/crypto.h>

#include <net/sadb.h>

/*
   Now it support algorithms MD5 and SHA1.
   It digest size is 64 byte. See RFC2401

   It should be a valiable value in ipsec_sa or digest context.


*/
#define DIGEST_SIZE 64


int calc_hmac(struct ipsec_sa *ipsec_sa, __u8 *in, int size, __u8 *hmac)
{
	int i = 0;
	int error = 0;
	int blocksize = 0;
	struct digest_implementation *di = NULL;
	int key_len = 0;
	__u8 *key = NULL;
	__u8 k_ipad[DIGEST_SIZE+1];    /* inner padding - key XORd with ipad */
	__u8 k_opad[DIGEST_SIZE+1];    /* outer padding - key XORd with opad */
	__u8 *tk = NULL;


	if(!(ipsec_sa&&in&&hmac)){
		pr_debug("calc_hmac: some parameter is null\n");
		error = -EINVAL;
		goto err;
	}

	if(!(di = ipsec_sa->auth_algo.cx.di)){
		pr_debug("calc_hmac: digest_implementation is null\n");
		error = -EINVAL;
		goto err;
	}

#if 0
	pr_debug("calc_hmac: blocksize=%d\n", di->blocksize);
	pr_debug("calc_hmac: key_len=%d\n", ipsec_sa->auth_algo.key_len);
	pr_debug("calc_hmac: size=%d\n", size);
#endif

	blocksize = di->blocksize;
	key_len = ipsec_sa->auth_algo.key_len;
	key = ipsec_sa->auth_algo.key;

#if 0
	printk("key:");
	for(i=0; i<key_len; i++){
		printk("%x", 0xFF & key[i]);
	}
	printk("\n");
#endif

	/* if key is longer than 64 bytes reset it to key=H(key) */
	if (key_len > DIGEST_SIZE) {

		tk = kmalloc(blocksize, GFP_KERNEL);
		if(!tk){
			pr_debug("calc_hmac: tk is not allocated\n");
			error = -ENOMEM;
			goto err;
		}

		memset(tk, 0, blocksize);

		di->open(&(ipsec_sa->auth_algo.cx));
		di->update(&(ipsec_sa->auth_algo.cx), key, key_len);
		di->close(&(ipsec_sa->auth_algo.cx), tk);
		key = tk;
		key_len = blocksize;

#if 0
		printk("key is showter than 64:");
		for(i=0; i<key_len; i++){
			printk("%x", 0xFF & tk[i]);
		}
		printk("\n");
#endif

		kfree(tk);
	}

	/*
	 * the HMAC_MD5 transform looks like:
	 *
	 * H(K XOR opad, H(K XOR ipad, text))
	 *
	 * where K is an n byte key
	 * ipad is the byte 0x36 repeated 64 times
	 * opad is the byte 0x5c repeated 64 times
	 * and text is the data being protected
	 */

	/* start out by storing key in pads */
	memset(k_ipad,0,sizeof(k_ipad));
	memset(k_opad,0,sizeof(k_opad));
	memcpy(k_ipad,key,key_len);
	memcpy(k_opad,key,key_len);

	/* XOR key with ipad and opad values */
	for (i=0; i<DIGEST_SIZE; i++) {
		k_ipad[i] ^= 0x36;
		k_opad[i] ^= 0x5c;
	}

#if 0
	printk("k_ipad:");
	for(i=0; i<DIGEST_SIZE; i++) {
		printk("%x", 0xFF & k_ipad[i]);
	}
	printk("\n");

	printk("k_opad:");
	for(i=0; i<DIGEST_SIZE; i++) {
		printk("%x", 0xFF & k_opad[i]);
	}
	printk("\n");
#endif
	/*
	 * perform inner
	 */
	di->open(&(ipsec_sa->auth_algo.cx));
	di->update(&(ipsec_sa->auth_algo.cx), k_ipad, DIGEST_SIZE);
	di->update(&(ipsec_sa->auth_algo.cx), in, size);
	di->close(&(ipsec_sa->auth_algo.cx), hmac);
#if 0
	printk("hmac_i:");
	for(i=0; i<ipsec_sa->auth_algo.digest_len; i++) {
		printk("%x", 0xFF & hmac[i]);
	}
	printk("\n");
#endif

	/*
	 * perform outer
	 */
	di->open(&(ipsec_sa->auth_algo.cx));
	di->update(&(ipsec_sa->auth_algo.cx), k_opad, DIGEST_SIZE);
	di->update(&(ipsec_sa->auth_algo.cx), hmac, blocksize);
	di->close(&(ipsec_sa->auth_algo.cx), hmac);
#if 0
	printk("hmac_o:");
	for(i=0; i<ipsec_sa->auth_algo.digest_len; i++) {
		printk("%x", 0xFF & hmac[i]);
	}
	printk("\n");
#endif

err:
	return error;

}

void calc_hmac_test()
{
	int i;
	char buf[64];
	__u32 digest_info[1024];
	struct ipsec_sa sa1;

	memset(&sa1, 0, sizeof(sa1));
	memset(buf, 0, 64);
	memset(digest_info, 0, 256);
	
	sa1.auth_algo.cx.di = find_digest_by_name("md5", 0);
	if(!(sa1.auth_algo.cx.di)){
               pr_debug("algorithm %s is not supported\n", "md5");
		return;
	}
#if 0
	pr_debug("blocksize=%u\n", sa1.auth_algo.cx.di->blocksize);
	pr_debug("working_size=%u\n", sa1.auth_algo.cx.di->working_size);
#endif

	sa1.auth_algo.cx.digest_info = digest_info;

	memcpy(sa1.auth_algo.key, "Jefe", 5);

	sa1.auth_algo.key_len = 4;
	sa1.auth_algo.digest_len = 16;

	calc_hmac(&sa1, "what do ya want for nothing?", 28, buf);

	for(i=0; i<64; i++){
		printk("%x", 0xFF & buf[i]);
	}
	printk("\n");

}



