/* $USAGI: spd_utils.c,v 1.12 2001/09/12 15:46:52 miyazawa Exp $ */

/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/* 
 * spd_utils provides utility routines for SPD processing.
 */

#define __NO_VERSION__ 
#include <linux/module.h>
#include <linux/version.h>
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/version.h>
#include <linux/types.h>

#include <linux/kernel.h>

#include <linux/ip.h>		/* struct iphdr */
#include <linux/ipv6.h>		/* struct ipv6hdr */
#include <linux/skbuff.h>

#include <net/pfkeyv2.h>
#include <net/sadb.h>
#include <net/spd.h>
#include "sadb_utils.h"

#define BUFSIZE 64

int compare_selector(struct selector *selector1, struct selector *selector2)
{
	int retval = 0;

	if (!(selector1&&selector2)) {
		pr_debug("compare_selector: selector1 or selecotr2 is null\n");
		return -EINVAL;
	}

	if (selector1->proto && selector2->proto) {
		if (selector1->proto != selector2->proto) 
			return -EINVAL;
	}

	return compare_address_with_prefix((struct sockaddr*)&(selector1->src), selector1->prefixlen_s,
					(struct sockaddr*)&(selector2->src), selector2->prefixlen_s)
		|| compare_address_with_prefix((struct sockaddr*)&(selector1->dst), selector1->prefixlen_d,
					(struct sockaddr*)&(selector2->dst), selector2->prefixlen_d);
}

void dump_ipsec_sp(struct ipsec_sp *policy)
{

	char buf[BUFSIZE];
	struct list_head *pos;
	struct sa_index  *tmp;

	if (!policy) {
		pr_debug("dump_spd_entry: policy is null\n");
		return;
	}

	memset(buf, 0, BUFSIZE);
	sockaddrtoa((struct sockaddr*)&((policy->selector).src), buf, BUFSIZE);
	pr_info("spd.selector.src=%s/%u\n", buf, policy->selector.prefixlen_s);

	memset(buf, 0, BUFSIZE);
	sockaddrtoa((struct sockaddr*)&((policy->selector).dst), buf, BUFSIZE);
	pr_info("spd.selector.dst=%s/%u\n", buf, policy->selector.prefixlen_d);
	
	pr_info("spd.action=%d\n", policy->action);

	list_for_each(pos, &(policy->sa_index_list)){
		tmp = list_entry(pos, struct sa_index, entry);
		if (!tmp) {
			pr_info("dump_spd_entry: tmp is null\n");
			continue;
		}
		memset(buf, 0, BUFSIZE);
		sockaddrtoa((struct sockaddr*)&(tmp->dst), buf, BUFSIZE);
		pr_info("spd.sa_index_list.dst=%s ", buf);
		pr_info("spd.sa_index_list.ipsec_proto=%u ", tmp->ipsec_proto);
		pr_info("spd.sa_index_list.spi=%u ", tmp->spi);	
		pr_info("spd.sa_index_list.sa=%p\n", tmp->sa);
	}
}

